#!/bin/sh


#/*************************************************************************
#**    Carl Zeiss MicroImaging GmbH
#**************************************************************************
#**
#**       File: autoupdate.sh
#**    Summary: update installer script for czsensor application binary
#**
#**    Version: 1.4
#**       Date: 07.07.2008
#**     Author: Johannes Paschold (johannes.paschold@zeiss.com)
#**     Author: Andreas Kirsch (andreas.kirsch@zeiss.com)
#**
#** History
#** 	- update to version 3X
#** 	- czsenor moved to /data and symbolic link in /usr/bin/czsensor
#** 	- references moved to /data and symbolic link in /usr/bin/czsensor
#** 	- tar added in /data and symbolic link in /bin
#** 10.07.2007
#**	- removing files before extracting them
#**	- output changed
#**	- /sbin added to $PATH
#** 22.08.07 
#**	- new smaller tar binary (original eldk 3.0)
#**	- added filtering of "timestamp is in the future error" (tar -xvz -f czsensor.tar.gz -C /tmp file 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
#**
#** 07.07.2008
#**	- script header
#**	- changed app version to 39
#** 06.04.2009
#**	- added inittab to tarball
#**	- added wait for sync
#**	- changed version to 1.2
#** 07.07.2011
#**	- added comments to prevent other developers wasting time on optimization
#**	- 7za added to install_tools()
#**	- busybox added to install_tools() but not activated because we need more checks if it works as upx compressed ELF
#** 20.06.2012
#**	- JD-3158 [CR] License dependency
#**	    - removed: dependency to Berkeley-DB (dependency of APR and APR-Utils)
#**	    - added:   new APR and APR-Utils
#**	    - changed: now we are using the standard shared library cache to distribute links to shared libraries
#** 11.01.2013
#**	- changed: some libraries updated
#**	- changed: JD-3481 [CR] Update v42 deletes all AMO files
#**	- removed: some comments
#**	- added: umountusb.sh containing workaround for JD-3503 [B] sometimes USB stick does not umount
#** 19.08.2013
#**	- changed: new mOLUP version
#**	- added: HTML Templates
#**	- changed: user rights corrected
#**	- removed: unnecessary JavaScript
#**************************************************************************



# main entry point
############################################################

export PATH=$PATH:/sbin
VERSION=1.3
APPVERSION=0
ARCHIVENAME='czsensorupdate.tar.gz'

USRLIBDIR='/usr/lib'
APPDIR='/usr/bin/czsensor'
REFERENCEDIR='/reference'
KERNEL='/lib/modules/3rdparty'
APACHE_DIR='/usr/bin/apache'
SRCDIR='/tmp/czsensorupdate'

J1939LIB='libJ1939.so.1.17'
SERPROTLIB='libserprot.so.1.1'
MCSOEMLIB='libmcsoem.so.1.1'
MOLUPLIB_OLD='libmobileolup.so'
MOLUPLIB='libmobileolup.so.2.1'
USBSPECLIB='libusbspect.so.0.0.0'
CGICCLIB='libcgicc.so.3.2.10'
CGICCLIB_OLD='libcgicc.so.5.0.1'
LOG4CXXLIB='liblog4cxx.so.9.0.7'
LOG4CXXLIB_OLD='liblog4cxx.so.9.0.2'
EXPATLIB='libexpat.so.0.5.0'
RTLIB='librt-2.3.1.so'
APACHEBIN='apache_image.tar'
DLLCAN='DLLcan.o'
DBLIB='libdb-4.0.so'
TAROPT='-xvz -f'
INITTAB='inittab'


show_free_diskspace()
{
	echo "available diskspace:"
	echo 
	df -h | grep Used
	df -h | grep block1
	echo
}


delete_unneeded()
{
# this can not be shortened because busybox does not support shell arrays
	echo
	echo "Checking for unneeded files"
	echo "--------------------------------"
	echo 
	rm -f /usr/bin/dll-*
	FOUNDANY=0
	
	FILENAME="/usr/lib/DLLcan.o"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi
	
	
	FILENAME="/lib/modules/3rdparty/dllcan.o"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi
	
	FILENAME="/usr/bin/dll-define-cob"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi
	
	FILENAME="/usr/bin/dll-demo"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi
	
	FILENAME="/usr/bin/dll-init"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi
	
	FILENAME="dll-receive"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi
	
	FILENAME="dll-send"
	if [ -f "$FILENAME" ]; then
	   echo "deleting $FILENAME"
	   rm $FILENAME
	   FOUNDANY=1
	fi

	if [ $FOUNDANY = 0 ]; then
	   echo "none."
	else
	   show_free_diskspace
	fi

}

delete_app_binaries()
{
# this can not be shortened because busybox does not support shell arrays
	echo "deleting czsensor"
	rm /data/czsensor
	rm /usr/bin/czsensor/czsensor
	
	echo "deleting libcz"
	rm /usr/lib/libcz*
	
	echo "deleting libHTML"
	rm /usr/lib/libHTML*
	
	echo "deleting all CGIs in $APACHECGIDIR"
	rm -f $APACHE_DIR/cgi-bin/*
}

install_app_binaries()
{
	echo
	echo "deploying application files"
	echo "---------------------------"
	echo
	cd /data

	#
	# czsensor application deployment
	#
	# todo: do only install if first deployment
	#(tar -xvzf czsensorupdate.tar.gz    -C /usr/bin/czsensor              czsensor.config   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $USRLIBDIR                     libcz.so          2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C /data                          czsensor          2>&1 1>&3 | grep -v "future" 1>&2) 3>&1

	rm /usr/bin/czsensor/czsensor
	ln -sf /data/czsensor /usr/bin/czsensor/czsensor
	chmod a+x /data/czsensor

	(tar -xvzf czsensorupdate.tar.gz    -C /etc                           inittab           2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	
	#
	# WebUI content deployment
	#
	rm -f   $APACHE_DIR/cgi-bin/*
	rm -f   $APACHE_DIR/htdocs/*.htm
	# don't clean the following folders because the apache image comes already with necessary files
	# rm -f   $APACHE_DIR/htdocs/images/*
	# rm -f   $APACHE_DIR/htdocs/css/*
	# rm -f   $APACHE_DIR/htdocs/js/*
	rm -f   $APACHE_DIR/htdocs/images/DE.png
	rm -f   $APACHE_DIR/htdocs/images/ES.png
	rm -f   $APACHE_DIR/htdocs/images/FR.png
	rm -f   $APACHE_DIR/htdocs/images/US.png
	rm -f   $APACHE_DIR/htdocs/css/CzScripts.js
	rm -f   $APACHE_DIR/htdocs/css/style.css
	
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          measurepage                               2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          measurementsettingspage                   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          systemsettingspage                        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          systemstatuspage                          2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          calibrationadministrationpage             2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          qv                                        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/cgi-bin          servicesettingspage                       2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/images/   progressBarAjaxLoader.gif                 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/images    DE.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/images    ES.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/images    FR.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/images    US.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/css       style.css                                 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs/js        CzScripts.js                              2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           calibrationadministrationpageTemplate.htm 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           measurementsettingspageTemplate.htm       2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           measurementsettingspageTemplateManure.htm 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           measurepageTemplate.htm                   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           servicesettingspageTemplate.htm           2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           systemsettingspageTemplate.htm            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           systemstatuspageTemplate.htm              2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           systemstatuspageTemplateManure.htm        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           qvTemplate.htm                            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           qvTemplateManure.htm                      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvzf czsensorupdate.tar.gz    -C $APACHE_DIR/htdocs           qvSelfTestTemplate.htm                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	
	#find $APACHE_DIR/htdocs/ -type d -print | xargs -0 chmod 555
	#find $APACHE_DIR/htdocs/ -type f -print | xargs -0 chmod 444
	find $APACHE_DIR/htdocs/ -type d | xargs -r chmod 555
	find $APACHE_DIR/htdocs/ -type f | xargs -r chmod 444
	chown -R http:http $APACHE_DIR/htdocs

	chmod -R 555       $APACHE_DIR/cgi-bin
	chown -R http:http $APACHE_DIR/cgi-bin
}

# JD-2188 [FR] get calibration information (crop, ingredient) from AMO file header
install_models()
{
	echo
	echo "installing new / updating old models if necessary"
	echo "----------------------------------"
	echo
	
	#
	# initial references deployment
	#
	if [ -f "/data/version" ]; then
	   echo "Firmware version found. No initial references deployment necessary."
	else
	   echo "No Firmware version found. Starting initial references deployment."
	   (tar -xvzf czsensorupdate.tar.gz    -C /usr/bin/czsensor/reference    externaldark.spc  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	   (tar -xvzf czsensorupdate.tar.gz    -C /usr/bin/czsensor/reference    externalwhite.spc 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	   (tar -xvzf czsensorupdate.tar.gz    -C /usr/bin/czsensor/reference    internalwhite.spc 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	   (tar -xvzf czsensorupdate.tar.gz    -C /usr/bin/czsensor/reference    internaldark.spc  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	fi

	cd /data
# JD-3481 [CR] Update v42 deletes all AMO files
	SW_VERSION=`cat /data/version`
	if [ $SW_VERSION -lt 42 ];
	then
		echo "Old Sensor Software detected (current SW Version '$SW_VERSION' < V42)."
		echo "--- Removing old model files..."
		rm -f /data/crop0/ingredient0/*.AMO
		rm -f /data/crop0/ingredient0/*.amo
		rm -f /data/crop0/ingredient0/*.ZSM
		rm -f /data/crop0/ingredient0/*.zsm
		rm -f /data/crop0/ingredient0/lastknownmodels.conf
		rm -f /data/crop0/ingredient0/lastknownmodels_HMS.conf
		rm -f /data/crop0/ingredient0/lastknownmodels_MCS.conf
		rm -f /data/crop0/inactive/*
	else if [ $SW_VERSION -gt 999 ];
		then
			echo "Test Sensor Software detected (current SW Version '$SW_VERSION' > V999)."
			echo "--- Removing old model files..."
			rm -f /data/crop0/ingredient0/*.AMO
			rm -f /data/crop0/ingredient0/*.amo
			rm -f /data/crop0/ingredient0/*.ZSM
			rm -f /data/crop0/ingredient0/*.zsm
			rm -f /data/crop0/ingredient0/lastknownmodels.conf
			rm -f /data/crop0/ingredient0/lastknownmodels_HMS.conf
			rm -f /data/crop0/ingredient0/lastknownmodels_MCS.conf
			rm -f /data/crop0/ActivationList.DRM
			rm -f /data/crop0/inactive/*
		else
			echo "Post V41 Sensor Software Version '$SW_VERSION' detected."
			echo "--- leaving recent model files untouched..."
		fi
	fi
	echo "--- Installing new model files..."
	rm -f /data/HarvestLabTranslations.xml
	(tar $TAROPT $ARCHIVENAME -C /data/crop0/ingredient0 --wildcards *.AMO 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /data/crop0/ingredient0 --wildcards *.zsm 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	echo invalMeasurementDataOnAmoFileDeact > /data/crop0/ingredient0/lastknownmodels.conf
	echo invalMeasurementDataOnAmoFileDeact > /data/crop0/ingredient0/lastknownmodels_HMS.conf
	echo invalMeasurementDataOnAmoFileDeact > /data/crop0/ingredient0/lastknownmodels_MCS.conf
	(tar $TAROPT $ARCHIVENAME -C /data HarvestLabTranslations.xml 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	# JD-2791 [FR] adjust measurement value (dry/wet)
	(tar $TAROPT $ARCHIVENAME -C /data/crop0/ingredient0 DryWet.conf 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	chown http:http /data/crop0/ingredient0/DryWet.conf
}


delete_libraries()
{
	FILENAME="$USRLIBDIR/$J1939LIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$SERPROTLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$MCSOEMLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$MOLUPLIB_OLD"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$MOLUPLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$USBSPECLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$CGICCLIB_OLD"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$CGICCLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$LOG4CXXLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="$USRLIBDIR/$DBLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="/lib/$DBLIB"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="/sbin/ldconfig"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
}

install_libraries()
{
	echo
	echo "installing libraries"
	echo "--------------------------------"
	echo
#	mkdir /data/lib
#	chmod 777 /data/lib
	# if /dev/mtdblock1 is full we can use /dev/mtdblock0 for libraries:
	touch /etc/ld.so.conf
	# fill the ldconfig file with the to be parsed paths like 
	#     echo '/data/lib' > /etc/ld.so.conf
	# or
	#     echo '/data/lib' >> /etc/ld.so.conf
	# while /lib and /usr/lib will be parsed automatically

	# remove unnecessary files from previous HLab SW versions
	rm -f /usr/lib/liblog4cxx*
	rm -f /usr/lib/libapr*
	rm -f /usr/lib/libaprutil*
	rm -f /usr/lib/libexpat*
	rm -f /usr/lib/libdb-4.0*
	rm -f /usr/lib/libJ1939*
	rm -f /usr/lib/libcz*
	rm -f /usr/lib/libcgicc*
	rm -f /usr/lib/libmobile*

	rm -f /lib/liblog4cxx*
	rm -f /lib/libapr*
	rm -f /lib/libexpat*
	rm -f /lib/libdb-4.0*

	# extract all necessary libraries
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR libACABase.so 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $J1939LIB     2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $SERPROTLIB   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $MCSOEMLIB    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $MOLUPLIB     2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $USBSPECLIB   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $CGICCLIB     2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $LOG4CXXLIB   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR $EXPATLIB     2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /lib       $RTLIB        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR libcz.so      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C $USRLIBDIR libHTML.so    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /sbin      ldconfig      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1

	# set rights for library access
	chmod 777 $USRLIBDIR/libACABase.so
	chmod 755 $USRLIBDIR/$MOLUPLIB

	# publish libraries to system
	ldconfig -v # be verbose
	ldconfig -l $USRLIBDIR/libACABase.so # Manually link individual libraries
	ldconfig -l $USRLIBDIR/libHTML.so
	ldconfig -l $USRLIBDIR/libcz.so
	ldconfig -l $USRLIBDIR/$MOLUPLIB

	ln -s $USRLIBDIR/$MOLUPLIB /usr/lib/libmobileolup.so
	
	show_free_diskspace
}

delete_candriver()
{
	rmmod DLLcan
	rm -f $KERNEL/$DLLCAN
}

install_candriver()
{
	echo
	echo "installing can driver"
	echo "--------------------------------"
	echo
	(tar $TAROPT $ARCHIVENAME -C $KERNEL $DLLCAN 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
}

delete_tools()
{
	FILENAME="/sbin/unlink"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	#keep the mountusb script
	#FILENAME="/usr/bin/mountusb.sh"
	#if [ -f $FILENAME ]; then
	#	echo "deleting $FILENAME"
	#	rm $FILENAME
	#fi
	FILENAME="/usr/bin/amocp"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
	FILENAME="/usr/bin/amoeval"
	if [ -f $FILENAME ]; then
		echo "deleting $FILENAME"
		rm $FILENAME
	fi
}

install_tools()
{
	echo
	echo "installing tools"
	echo "--------------------------------"
	echo
	(tar $TAROPT $ARCHIVENAME -C /usr/bin 7za          2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /bin     unlink       2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /usr/bin mountusb.sh  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /usr/bin umountusb.sh 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /usr/bin mountNFS.sh  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /usr/bin amocp        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar $TAROPT $ARCHIVENAME -C /usr/bin amoeval      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
#	(tar $TAROPT $ARCHIVENAME -C /bin     busybox      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
#	chmod u+s /bin/busybox
	#add new user passwords
	(tar $TAROPT $ARCHIVENAME -C /etc     passwd      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
#
# Development Tools like syslogd via /etc/rc.sh and /etc/syslog.conf
#
#	(tar $TAROPT $ARCHIVENAME -C /etc     rc.sh       2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
#	(tar $TAROPT $ARCHIVENAME -C /etc     syslog.conf 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1

	chmod 777 /usr/bin/7za
}

extract_apache()
{
	echo
	echo "installing APACHE webserver"
	echo "--------------------------------"
	echo
	cd /data
	(tar -xvz -f $ARCHIVENAME    -C /tmp $APACHEBIN 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
        (tar -x   -f /tmp/$APACHEBIN -C /tmp            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
}

delete_apache()
{
	FILENAME="/usr/bin/apache"
	if [ -d $FILENAME ]; then
		echo "recursively deleting $FILENAME"
		rm -rf $FILENAME
	fi
}

move_apache()
{
	mv /tmp/apache /usr/bin
	cd /usr/bin/apache
	ln -s -f /var/tmp logs
	rm /tmp/$APACHEBIN
}

printhelp()
{
	echo
	echo "SPFH update script, v$VERSION"
	echo
	echo "Usage update [PARAM]"
	echo
	echo "	--hlp			This screen"
	echo "	--version		Print version number of update script and exit"
	echo "	--deleteunneeded	Remove unneeded files in rootfs and exit"
	echo "	--uninstall		Remove all application files and exit"
	echo "  --prepatchonly		Just apply prepatch.tar.gz and exit"
	#echo "	--withmodels		Run update and install models"
	#echo "	--withfilters		Run update and install models and filters"
	echo 
}

apply_prepatch()
{
	if [ -f "/data/prepatch" ]; then

		echo
		echo "applying prepatch"
		echo "--------------------------------"
		echo

		/data/./prepatch
		if [ ${?} = 0 ]; then
		   echo
		   echo "prepatch successfull"
		else
		   echo
		   echo "prepatch failed"
		fi

		echo
	fi
}

get_app_version()
{
	if [ -f "/data/version" ]; then
    		APPVERSION=$(cat /data/version)
	fi
	if [ -f "/tmp/version" ]; then
    		APPVERSION=$(cat /tmp/version)
	fi
	if [ $APPVERSION = 0 ]; then
		echo "Application version: unknown"
	else
		echo
		echo "Application version: $APPVERSION"
	fi
}

finalize()
{
	echo
	echo "finalizing"
	echo "--------------------------------"
	echo
	echo "deleting $ARCHIVENAME"
	rm /data/$ARCHIVENAME
	echo "deleting autoupdate.sh"
	rm /data/autoupdate.sh
	echo "wait for sync..."
	sync
	echo "ok"
}




#
# first, check userid

if [ $(whoami) != "root" ]; then
    echo
    echo "ERROR: Please run as root."
    echo
    exit
fi

#
# check parameters

if [ "$1" = "--help" ]; then
   printhelp
   exit
fi

if [ "$1" = "--version" ]; then
   echo $VERSION
   exit
fi

if [ "$1" = "--deleteunneeded" ]; then
   delete_unneeded
   echo
   exit
fi

if [ "$1" = "--uninstall" ]; then
   delete_unneeded
   killall czsensor
   delete_app_binaries
   delete_libraries
   rmmod -a
   delete_candriver
   delete_tools
   echo
   exit
fi

if [ "$1" = "--prepatchonly" ]; then
   apply_prepatch
   exit
fi




# start normal update procedure
############################################################

echo
echo "starting update..."
echo 

apply_prepatch
get_app_version
show_free_diskspace

# on some prototypes there are unneeded files which 
# waste flash space
delete_unneeded

cd /data

extract_apache
delete_apache
move_apache
install_app_binaries
install_models
install_libraries
install_candriver
install_tools
finalize

echo
echo "finished update, rebooting..."
/sbin/reboot
echo
